/*
 * Copyright (c) 2024. Das Land Schleswig-Holstein vertreten durch das Ministerium für Energiewende, Klimaschutz, Umwelt und Natur
 * Zentrales IT-Management
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.admin.common.errorhandling;

import java.util.ArrayList;
import java.util.Collections;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.Set;

import jakarta.validation.ConstraintViolation;
import jakarta.validation.ConstraintViolationException;

import org.springframework.data.rest.webmvc.ResourceNotFoundException;
import org.springframework.http.HttpStatus;
import org.springframework.http.ProblemDetail;
import org.springframework.security.access.AccessDeniedException;
import org.springframework.web.ErrorResponse;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.RestControllerAdvice;
import org.springframework.web.servlet.mvc.method.annotation.ResponseEntityExceptionHandler;

import de.ozgcloud.common.errorhandling.TechnicalException;

@RestControllerAdvice
public class ExceptionController extends ResponseEntityExceptionHandler {

	@ExceptionHandler(RuntimeException.class)
	public ErrorResponse handleRuntimeException(RuntimeException ex) {
		return ErrorResponse.builder(ex, HttpStatus.INTERNAL_SERVER_ERROR, ex.getLocalizedMessage()).build();
	}

	@ExceptionHandler(AccessDeniedException.class)
	public ErrorResponse handleAccessDeniedException(AccessDeniedException ex) {
		return ErrorResponse.builder(ex, HttpStatus.FORBIDDEN, ex.getLocalizedMessage()).build();
	}

	@ExceptionHandler(ResourceNotFoundException.class)
	public ErrorResponse handleResourceNotFoundException(ResourceNotFoundException ex) {
		return ErrorResponse.builder(ex, HttpStatus.NOT_FOUND, ex.getLocalizedMessage()).build();
	}

	@ExceptionHandler(FunctionalException.class)
	public ErrorResponse handleFunctionalException(FunctionalException ex) {
		return ErrorResponse.builder(ex, HttpStatus.BAD_REQUEST, ex.getLocalizedMessage()).build();
	}

	@ExceptionHandler(TechnicalException.class)
	public ErrorResponse handleTechnicalException(TechnicalException ex) {
		return ErrorResponse.builder(ex, HttpStatus.INTERNAL_SERVER_ERROR, ex.getLocalizedMessage()).build();
	}

	@ExceptionHandler(ConstraintViolationException.class)
	public ErrorResponse handleConstraintViolationException(ConstraintViolationException ex) {
		var problemDetail = buildProblemDetail(HttpStatus.UNPROCESSABLE_ENTITY, ex);
		return ErrorResponse.builder(ex, problemDetail).build();
	}

	private ProblemDetail buildProblemDetail(HttpStatus status, ConstraintViolationException ex) {
		var problemDetail = ProblemDetail.forStatusAndDetail(status, ex.getLocalizedMessage());
		problemDetail.setProperty("invalid-params", getDetailedviolationList(ex.getConstraintViolations()));
		return problemDetail;
	}

	private List<Map<String, String>> getDetailedviolationList(Set<ConstraintViolation<?>> violations) {
		List<Map<String, String>> detailedViolations = new ArrayList<>();
		Optional.ofNullable(violations).orElse(Collections.emptySet()).forEach(v -> detailedViolations.add(buildDetailedViolation(v)));
		return detailedViolations;

	}

	private Map<String, String> buildDetailedViolation(ConstraintViolation<?> violation) {
		var detailedViolation = new LinkedHashMap<String, String>();
		detailedViolation.put("name", violation.getPropertyPath().toString());
		detailedViolation.put("reason", violation.getMessage());
		return detailedViolation;
	}
}
